@@ -16,18 +16,29 @@
 from __future__ import annotations
 
 from collections import deque
-from typing import TYPE_CHECKING, Any, Generic, Iterator, Mapping, NoReturn, Optional
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Generic,
+    Iterator,
+    Mapping,
+    NoReturn,
+    Optional,
+    Sequence,
+    Union,
+)
 
-from bson import _convert_raw_document_lists_to_streams
-from pymongo.cursor import _CURSOR_CLOSED_ERRORS, _SocketManager
+from bson import CodecOptions, _convert_raw_document_lists_to_streams
+from pymongo.cursor import _CURSOR_CLOSED_ERRORS, _ConnectionManager
 from pymongo.errors import ConnectionFailure, InvalidOperation, OperationFailure
-from pymongo.message import _CursorAddress, _GetMore, _RawBatchGetMore
+from pymongo.message import _CursorAddress, _GetMore, _OpMsg, _OpReply, _RawBatchGetMore
 from pymongo.response import PinnedResponse
-from pymongo.typings import _Address, _DocumentType
+from pymongo.typings import _Address, _DocumentOut, _DocumentType
 
 if TYPE_CHECKING:
     from pymongo.client_session import ClientSession
     from pymongo.collection import Collection
+    from pymongo.pool import Connection
 
 
 class CommandCursor(Generic[_DocumentType]):
@@ -51,7 +62,9 @@ class CommandCursor(Generic[_DocumentType]):
         self.__collection: Collection[_DocumentType] = collection
         self.__id = cursor_info["id"]
         self.__data = deque(cursor_info["firstBatch"])
-        self.__postbatchresumetoken = cursor_info.get("postBatchResumeToken")
+        self.__postbatchresumetoken: Optional[Mapping[str, Any]] = cursor_info.get(
+            "postBatchResumeToken"
+        )
         self.__address = address
         self.__batch_size = batch_size
         self.__max_await_time_ms = max_await_time_ms
@@ -60,9 +73,9 @@ class CommandCursor(Generic[_DocumentType]):
         self.__killed = self.__id == 0
         self.__comment = comment
         if self.__killed:
-            self.__end_session(True)
+            self.__end_session()
 
-        if "ns" in cursor_info:
+        if "ns" in cursor_info:  # noqa: SIM401
             self.__ns = cursor_info["ns"]
         else:
             self.__ns = collection.full_name
@@ -75,12 +88,13 @@ class CommandCursor(Generic[_DocumentType]):
     def __del__(self) -> None:
         self.__die()
 
-    def __die(self, synchronous=False):
+    def __die(self, synchronous: bool = False) -> None:
         """Closes this cursor."""
         already_killed = self.__killed
         self.__killed = True
         if self.__id and not already_killed:
             cursor_id = self.__id
+            assert self.__address is not None
             address = _CursorAddress(self.__address, self.__ns)
         else:
             # Skip killCursors.
@@ -98,16 +112,16 @@ class CommandCursor(Generic[_DocumentType]):
             self.__session = None
         self.__sock_mgr = None
 
-    def __end_session(self, synchronous):
+    def __end_session(self) -> None:
         if self.__session and not self.__explicit_session:
-            self.__session._end_session(lock=synchronous)
+            self.__session.end_session()
             self.__session = None
 
     def close(self) -> None:
         """Explicitly close / kill this cursor."""
         self.__die(True)
 
-    def batch_size(self, batch_size: int) -> "CommandCursor[_DocumentType]":
+    def batch_size(self, batch_size: int) -> CommandCursor[_DocumentType]:
         """Limits the number of documents returned in one batch. Each batch
         requires a round trip to the server. It can be adjusted to optimize
         performance and limit data transfer.
@@ -120,8 +134,7 @@ class CommandCursor(Generic[_DocumentType]):
         Raises :exc:`TypeError` if `batch_size` is not an integer.
         Raises :exc:`ValueError` if `batch_size` is less than ``0``.
 
-        :Parameters:
-          - `batch_size`: The size of each batch of results requested.
+        :param batch_size: The size of each batch of results requested.
         """
         if not isinstance(batch_size, int):
             raise TypeError("batch_size must be an integer")
@@ -131,34 +144,34 @@ class CommandCursor(Generic[_DocumentType]):
         self.__batch_size = batch_size == 1 and 2 or batch_size
         return self
 
-    def _has_next(self):
+    def _has_next(self) -> bool:
         """Returns `True` if the cursor has documents remaining from the
         previous batch.
         """
         return len(self.__data) > 0
 
     @property
-    def _post_batch_resume_token(self):
+    def _post_batch_resume_token(self) -> Optional[Mapping[str, Any]]:
         """Retrieve the postBatchResumeToken from the response to a
         changeStream aggregate or getMore.
         """
         return self.__postbatchresumetoken
 
-    def _maybe_pin_connection(self, sock_info):
+    def _maybe_pin_connection(self, conn: Connection) -> None:
         client = self.__collection.database.client
         if not client._should_pin_cursor(self.__session):
             return
         if not self.__sock_mgr:
-            sock_info.pin_cursor()
-            sock_mgr = _SocketManager(sock_info, False)
+            conn.pin_cursor()
+            conn_mgr = _ConnectionManager(conn, False)
             # Ensure the connection gets returned when the entire result is
             # returned in the first batch.
             if self.__id == 0:
-                sock_mgr.close()
+                conn_mgr.close()
             else:
-                self.__sock_mgr = sock_mgr
+                self.__sock_mgr = conn_mgr
 
-    def __send_message(self, operation):
+    def __send_message(self, operation: _GetMore) -> None:
         """Send a getmore message and handle the response."""
         client = self.__collection.database.client
         try:
@@ -169,8 +182,11 @@ class CommandCursor(Generic[_DocumentType]):
             if exc.code in _CURSOR_CLOSED_ERRORS:
                 # Don't send killCursors because the cursor is already closed.
                 self.__killed = True
-            # Return the session and pinned connection, if necessary.
-            self.close()
+            if exc.timeout:
+                self.__die(False)
+            else:
+                # Return the session and pinned connection, if necessary.
+                self.close()
             raise
         except ConnectionFailure:
             # Don't send killCursors because the cursor is already closed.
@@ -184,7 +200,7 @@ class CommandCursor(Generic[_DocumentType]):
 
         if isinstance(response, PinnedResponse):
             if not self.__sock_mgr:
-                self.__sock_mgr = _SocketManager(response.socket_info, response.more_to_come)
+                self.__sock_mgr = _ConnectionManager(response.conn, response.more_to_come)
         if response.from_command:
             cursor = response.docs[0]["cursor"]
             documents = cursor["nextBatch"]
@@ -192,6 +208,7 @@ class CommandCursor(Generic[_DocumentType]):
             self.__id = cursor["id"]
         else:
             documents = response.docs
+            assert isinstance(response.data, _OpReply)
             self.__id = response.data.cursor_id
 
         if self.__id == 0:
@@ -199,11 +216,16 @@ class CommandCursor(Generic[_DocumentType]):
         self.__data = deque(documents)
 
     def _unpack_response(
-        self, response, cursor_id, codec_options, user_fields=None, legacy_response=False
-    ):
+        self,
+        response: Union[_OpReply, _OpMsg],
+        cursor_id: Optional[int],
+        codec_options: CodecOptions[Mapping[str, Any]],
+        user_fields: Optional[Mapping[str, Any]] = None,
+        legacy_response: bool = False,
+    ) -> Sequence[_DocumentOut]:
         return response.unpack_response(cursor_id, codec_options, user_fields, legacy_response)
 
-    def _refresh(self):
+    def _refresh(self) -> int:
         """Refreshes the cursor with more data from the server.
 
         Returns the length of self.__data after refresh. Will exit early if
@@ -292,7 +314,7 @@ class CommandCursor(Generic[_DocumentType]):
 
     __next__ = next
 
-    def _try_next(self, get_more_allowed):
+    def _try_next(self, get_more_allowed: bool) -> Optional[_DocumentType]:
         """Advance the cursor blocking for at most one getMore command."""
         if not len(self.__data) and not self.__killed and get_more_allowed:
             self._refresh()
@@ -301,7 +323,25 @@ class CommandCursor(Generic[_DocumentType]):
         else:
             return None
 
-    def __enter__(self) -> "CommandCursor[_DocumentType]":
+    def try_next(self) -> Optional[_DocumentType]:
+        """Advance the cursor without blocking indefinitely.
+
+        This method returns the next document without waiting
+        indefinitely for data.
+
+        If no document is cached locally then this method runs a single
+        getMore command. If the getMore yields any documents, the next
+        document is returned, otherwise, if the getMore returns no documents
+        (because there is no additional data) then ``None`` is returned.
+
+        :return: The next document or ``None`` when no document is available
+          after running a single getMore or when the cursor is closed.
+
+        .. versionadded:: 4.5
+        """
+        return self._try_next(get_more_allowed=True)
+
+    def __enter__(self) -> CommandCursor[_DocumentType]:
         return self
 
     def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
@@ -342,15 +382,20 @@ class RawBatchCommandCursor(CommandCursor, Generic[_DocumentType]):
             comment,
         )
 
-    def _unpack_response(
-        self, response, cursor_id, codec_options, user_fields=None, legacy_response=False
-    ):
+    def _unpack_response(  # type: ignore[override]
+        self,
+        response: Union[_OpReply, _OpMsg],
+        cursor_id: Optional[int],
+        codec_options: CodecOptions,
+        user_fields: Optional[Mapping[str, Any]] = None,
+        legacy_response: bool = False,
+    ) -> list[Mapping[str, Any]]:
         raw_response = response.raw_response(cursor_id, user_fields=user_fields)
         if not legacy_response:
             # OP_MSG returns firstBatch/nextBatch documents as a BSON array
             # Re-assemble the array of documents into a document stream
             _convert_raw_document_lists_to_streams(raw_response[0])
-        return raw_response
+        return raw_response  # type: ignore[return-value]
 
     def __getitem__(self, index: int) -> NoReturn:
         raise InvalidOperation("Cannot call __getitem__ on RawBatchCursor")
